@@ -123,7 +123,7 @@ func TestTreeDecodingShaBoundary(t *testing.T) {
 func TestTreeMergeReplaceElements(t *testing.T) {
 	e1 := &TreeEntry{Name: "a", Filemode: 0100644, Oid: []byte{0x1}}
 	e2 := &TreeEntry{Name: "b", Filemode: 0100644, Oid: []byte{0x2}}
-	e3 := &TreeEntry{Name: "c", Filemode: 0100644, Oid: []byte{0x3}}
+	e3 := &TreeEntry{Name: "c", Filemode: 0100755, Oid: []byte{0x3}}
 
 	e4 := &TreeEntry{Name: "b", Filemode: 0100644, Oid: []byte{0x4}}
 	e5 := &TreeEntry{Name: "c", Filemode: 0100644, Oid: []byte{0x5}}
@@ -157,6 +157,7 @@ func TestMergeInsertElementsInSubtreeOrder(t *testing.T) {
 	assert.True(t, bytes.Equal(t1.Entries[1].Oid, []byte{0x2}))
 	assert.True(t, bytes.Equal(t1.Entries[2].Oid, []byte{0x3}))
 
+	require.Len(t, t2.Entries, 4)
 	assert.True(t, bytes.Equal(t2.Entries[0].Oid, []byte{0x4}))
 	assert.True(t, bytes.Equal(t2.Entries[1].Oid, []byte{0x1}))
 	assert.True(t, bytes.Equal(t2.Entries[2].Oid, []byte{0x2}))
@@ -166,9 +167,10 @@ func TestMergeInsertElementsInSubtreeOrder(t *testing.T) {
 type TreeEntryTypeTestCase struct {
 	Filemode int32
 	Expected ObjectType
+	IsLink bool
 }
 
-func (c *TreeEntryTypeTestCase) Assert(t *testing.T) {
+func (c *TreeEntryTypeTestCase) AssertType(t *testing.T) {
 	e := &TreeEntry{Filemode: c.Filemode}
 
 	got := e.Type()
@@ -177,14 +179,24 @@ func (c *TreeEntryTypeTestCase) Assert(t *testing.T) {
 		"gitobj: expected type: %s, got: %s", c.Expected, got)
 }
 
+func (c *TreeEntryTypeTestCase) AssertIsLink(t *testing.T) {
+	e := &TreeEntry{Filemode: c.Filemode}
+
+	isLink := e.IsLink()
+
+	assert.Equal(t, c.IsLink, isLink,
+		"gitobj: expected link: %v, got: %v, for type %s", c.IsLink, isLink, c.Expected)
+}
+
 func TestTreeEntryTypeResolution(t *testing.T) {
 	for desc, c := range map[string]*TreeEntryTypeTestCase{
-		"blob":    {0100644, BlobObjectType},
-		"subtree": {040000, TreeObjectType},
-		"symlink": {0120000, BlobObjectType},
-		"commit":  {0160000, CommitObjectType},
+		"blob":    {0100644, BlobObjectType, false},
+		"subtree": {040000, TreeObjectType, false},
+		"symlink": {0120000, BlobObjectType, true},
+		"commit":  {0160000, CommitObjectType, false},
 	} {
-		t.Run(desc, c.Assert)
+		t.Run(desc, c.AssertType)
+		t.Run(desc, c.AssertIsLink)
 	}
 }
 
